-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Expo: EvalTracker")
local dewdrop = AceLibrary("Dewdrop-2.0")

L:RegisterTranslations("enUS", function() return {
	["Eval Tracker"] = true,
	["Configure..."] = true,
	['New Eval...'] = true,
	['Delete Eval...'] = true,
	['No Evals Currently Tracked'] = true,
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local db
local buffer
local table_remove = table.remove

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Expo:NewModule("EvalTracker")
mod.revision = tonumber(("$Revision: 32680 $"):sub(12, -3)) or 1
mod.version = 1

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
end

function mod:OnModuleEnable()
	-- set up a database to store event filters, pause/resume status, and max buffer size.
	self.db = Expo:AcquireDBNamespace("EvalTracker")
	db = self.db.account
	db.evals = db.evals or {}
	if buffer then self:del(buffer) end
	buffer = self:new()
	-- Re-Eval the functions.
	for i,eval in ipairs(db.evals) do
		local eval_i,syntaxErr = loadstring("return "..eval[1])
		if eval_i then
			eval[2] = function()
				return self:new(pcall(eval_i))
			end
		else
			eval[2] = function()
				local ret = self:new()
				ret[2] = 'Syntax Error: '..syntaxErr
				return ret
			end
		end
	end
	self.core:UpdateTooltip()
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Eval Tracker"] end

function mod:TrackNewEval(newEval)
	local eval_i,syntaxErr = loadstring("return "..newEval)
	if eval_i then
		db.evals[#db.evals+1] = {
			newEval,function()
				return self:new(pcall(eval_i))
			end}
	else
		db.evals[#db.evals+1] = {
			newEval,function()
				local ret = self:new()
				ret[2] = 'Syntax Error: '..syntaxErr
				return ret
			end}
	end
	self.core:UpdateTooltip()
end

function mod:UntrackEval(evalIndex)
	table_remove(db.evals,evalIndex)
	self.core:UpdateTooltip()
end

function mod:GetNumberOfColumns() return 2 end
function mod:GetData(cat)
	-- add line to add and remove events
	cat:AddLine(
		'text', '',
		'text2', L["Configure..."],
		'justify2', 'RIGHT',
		'text2R', 0.6,
		'text2G', 0.6,
		'text2B', 1,
		'func',function() self:PopupConfigureMenu() end
	)
	for i, eval in ipairs(db.evals) do
		local rets = eval[2]()
		if rets[1] then
			rets[2] = self.core:FormatValue(rets[2])
			cat:AddLine('text',eval[1],'text2',rets[2])
			for i = 3, #rets do
				rets[i] = self.core:FormatValue(rets[i])
				cat:AddLine('text','','text2',rets[i])
			end
		else
			cat:AddLine('text',eval[1],'text2',rets[2],'text2R',1,'text2G',0.2,'text2B',0.2)
		end
		self:del(rets)
	end
end

function mod:PopupConfigureMenu()
	if not dewdrop:IsOpen("Expo_EvalTracker") then
		dewdrop:Close()
		dewdrop:Open("Expo_EvalTracker",'children',function(level,v,v_1,v_2,v_3,v_4)
				if level == 1 then
					dewdrop:AddLine('text',L['New Eval...'],
						'hasArrow',true,
						'hasEditBox',true,
						'editBoxFunc',self.TrackNewEval,'editBoxArg1',self)
					dewdrop:AddLine('text',L['Delete Eval...'],
						'hasArrow',true,'value','DeleteEvals')
				elseif level == 2 and v == 'DeleteEvals' then
					local no = true
					for i,eval in ipairs(db.evals) do
						dewdrop:AddLine('text',eval[1],
							'func',self.UntrackEval,'arg1',self,'arg2',i)
						no = nil
					end
					if no then
						dewdrop:AddLine('text',L['No Evals Currently Tracked'])
					end
				end
			end,
			'cursorX',true,'cursorY',true)
	else
		dewdrop:Close()
	end
end
