-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Expo: EventTracker")
local dewdrop = AceLibrary("Dewdrop-2.0")

L:RegisterTranslations("enUS", function() return {
	["Event Tracker"] = true,
	["Configure..."] = true,
	['New Event Track...'] = true,
	['Delete Event...'] = true,
	['No Events Currently Tracked'] = true,
	['Maximum Number of Events Displayed'] = true,
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local db
local buffer
local table_remove = table.remove

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Expo:NewModule("EventTracker")
mod.revision = tonumber(("$Revision: 32664 $"):sub(12, -3)) or 1
mod.version = 1

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
end

function mod:OnModuleEnable()
	-- set up a database to store event filters, pause/resume status, and max buffer size.
	self.db = Expo:AcquireDBNamespace("EventTracker")
	db = self.db.account
	db.events = db.events or {}
	db.maxBuffer = db.maxBuffer or 50
	if buffer then self:del(buffer) end
	buffer = self:new()
	-- set up event tracking
	--for event in pairs(db.events) do
		--self:RegisterEvent(event,"OnEvent")
	--end
	self:RegisterAllEvents("OnEvent")
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Event Tracker"] end

function mod:TrackNewEvent(newEvent)
	if not db.events[newEvent] then
		db.events[newEvent] = true
		--self:RegisterEvent(newEvent,"OnEvent")
	end
end

function mod:UntrackEvent(event)
	if db.events[event] then
		db.events[event] = nil
		--self:UnregisterEvent(event)
	end
end

local match

function mod:OnEvent(...)
	ev = AceLibrary("AceEvent-2.0").currentEvent
	for k in pairs(db.events) do
		match = string.match(ev,k)
		if match then break end
	end
	if not match then return end
	match = nil
	local event = self:new()
	event.event = ev
	local curtime = GetTime()
	local s = curtime
	local m = math.floor(math.floor(s) / 60)
	local h = math.floor(m / 60)
	s = s - 60 * m
	m = m - 60 * h
	h = h % 100
	event.time = string.format("%.2d:%.2d:%06.3f", h, m, s)
	event.rawtime = event.time..' ('..curtime..')'
	for i = 1, select("#",...) do local arg = (select(i,...))
		arg = self.core:FormatValue(arg)
		if arg then
			event[i] = arg
		end
	end
	while #buffer >= db.maxBuffer do
		local preEvent = buffer[1]
		table_remove(buffer,1)
		self:del(preEvent)
	end
	buffer[#buffer+1] = event
	self.core:UpdateTooltip()
end

local curttip

function mod:GetNumberOfColumns() return 2 end
function mod:GetData(cat)
	-- add line to add and remove events
	cat:AddLine(
		'text', '',
		'text2', L["Configure..."],
		'justify2', 'RIGHT',
		'text2R', 0.6,
		'text2G', 0.6,
		'text2B', 1,
		'func',function() self:PopupConfigureMenu() end
	)
	for i, event in ipairs(buffer) do
		local event = event
		cat:AddLine('text',event.time,'text2',event.event,'func',function() end,'onEnterFunc',function()
				GameTooltip:SetOwner(this, "ANCHOR_NONE")
				GameTooltip:SetPoint("TOPLEFT", this, "TOPRIGHT", 5, 0)
				curttip = self
				GameTooltip:ClearLines()
				GameTooltip:AddDoubleLine(event.rawtime,event.event,0.8,0.8,0.8,1,1,1)
				for i,arg in ipairs(event) do
					GameTooltip:AddDoubleLine('arg'..i..': ',arg,0.8,0.8,0.8,0.8,0.8,1)
				end
				GameTooltip:Show()
			end,'onLeaveFunc',function()
				if curttip == self then
					GameTooltip:Hide()
					curhlink = nil
				end
			end)
	end
end

function mod:PopupConfigureMenu()
	if not dewdrop:IsOpen("Expo_EventTracker") then
		dewdrop:Close()
		dewdrop:Open("Expo_EventTracker",'children',function(level,v,v_1,v_2,v_3,v_4)
				if level == 1 then
					dewdrop:AddLine('text',L['Maximum Number of Events Displayed'],
						'hasArrow',true,
						'hasEditBox',true,
						'editBoxText',db.maxBuffer,
						'editBoxFunc',function(self,newMax) db.maxBuffer = math.floor(tonumber(newMax)) end,'editBoxArg1',self)
					dewdrop:AddLine('text',L['New Event Track...'],
						'hasArrow',true,
						'hasEditBox',true,
						'editBoxFunc',self.TrackNewEvent,'editBoxArg1',self)
					dewdrop:AddLine('text',L['Delete Event...'],
						'hasArrow',true,'value','DeleteEvents')
				elseif level == 2 and v == 'DeleteEvents' then
					local no = true
					for ev in pairs(db.events) do
						dewdrop:AddLine('text',ev,
							'func',self.UntrackEvent,'arg1',self,'arg2',ev)
						no = nil
					end
					if no then
						dewdrop:AddLine('text',L['No Events Currently Tracked'])
					end
				end
			end,
			'cursorX',true,'cursorY',true)
	else
		dewdrop:Close()
	end
end
