-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Expo: PeriodicRefresh")
local dewdrop = AceLibrary("Dewdrop-2.0")

L:RegisterTranslations("enUS", function() return {
	["Periodic Refresh"] = true,
	["Configure..."] = true,
	["Not Refreshing."] = true,
	["Refreshing every second."] = true,
	["Refreshing every %s seconds."] = true,
	['Refresh Time...'] = true,
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local db
local buffer
local table_remove = table.remove

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Expo:NewModule("PeriodicRefresh")
mod.revision = tonumber(("$Revision: 32663 $"):sub(12, -3)) or 1
mod.version = 1

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
end

function mod:OnModuleEnable()
	self.db = Expo:AcquireDBNamespace("PeriodicRefresh")
	db = self.db.account
	db.refreshTime = db.refreshTime or 1
	self:SetRefreshSpeed(db.refreshTime)
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Periodic Refresh"] end

local refreshId

function mod:SetRefreshSpeed(newSpeed)
	newSpeed = tonumber(newSpeed)
	db.refreshTime = newSpeed
	if refreshId then
		self:CancelScheduledEvent(refreshId)
	end
	if newSpeed > 0 then
		refreshId = self:ScheduleRepeatingEvent(self.core.UpdateTooltip,newSpeed,self.core)
	else
		refreshId = nil
	end
end

function mod:GetNumberOfColumns() return 1 end
function mod:GetData(cat)
	-- add line to add and remove events
	cat:AddLine(
		'text', L["Configure..."],
		'justify', 'RIGHT',
		'textR', 0.6,
		'textG', 0.6,
		'textB', 1,
		'size', 12,
		'func',function() self:PopupConfigureMenu() end
	)
end

function mod:PopupConfigureMenu()
	if not dewdrop:IsOpen("Expo_PeriodicRefresh") then
		dewdrop:Close()
		dewdrop:Open("Expo_PeriodicRefresh",'children',function(level,v,v_1,v_2,v_3,v_4)
				if db.refreshTime <= 0 then
					dewdrop:AddLine('text',L["Not Refreshing."])
				elseif db.refreshTime == 1 then
					dewdrop:AddLine('text',L["Refreshing every second."])
				else
					dewdrop:AddLine('text',string.format(L["Refreshing every %s seconds."],tostring(db.refreshTime)))
				end
				dewdrop:AddLine('text',L['Refresh Time...'],
					'hasArrow',true,
					'hasEditBox',true,
					'editBoxFunc',self.SetRefreshSpeed,'editBoxArg1',self)
			end,
			'cursorX',true,'cursorY',true)
	else
		dewdrop:Close()
	end
end
